﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace C4F_Ocean
{
    /// <summary>
    /// A class to draw an ocean
    /// </summary>
    class Ocean
    {
        // the ocean's required content
        private Effect oceanEffect;
        private Texture2D[] OceanNormalMaps;

        // The two-triangle generated model for the ocean
        private VertexPositionNormalTexture[] OceanVerts;
        private VertexDeclaration OceanVD;

        /// <summary>
        /// Creates an Ocean object
        /// </summary>
        public Ocean()
        {

        }

        /// <summary>
        /// Loads the content necessary for the ocean
        /// </summary>
        /// <param name="Content"></param>
        public void Load(ContentManager Content)
        {
            // load the shader
            oceanEffect = Content.Load<Effect>("OceanShader");
            // load the normal maps
            OceanNormalMaps = new Texture2D[4];
            for (int i = 0; i < 4; i++)
                OceanNormalMaps[i] = Content.Load<Texture2D>("Ocean" + (i + 1) + "_N");

            // generate the geometry
            OceanVerts = new VertexPositionNormalTexture[6];
            OceanVerts[0] = new VertexPositionNormalTexture(new Vector3(-1000, 0, -1000), Vector3.Up, new Vector2(0, 0));
            OceanVerts[1] = new VertexPositionNormalTexture(new Vector3( 1000, 0, -1000), Vector3.Up, new Vector2(1, 0));
            OceanVerts[2] = new VertexPositionNormalTexture(new Vector3(-1000, 0,  1000), Vector3.Up, new Vector2(0, 1));
            OceanVerts[3] = OceanVerts[2];
            OceanVerts[4] = OceanVerts[1];
            OceanVerts[5] = new VertexPositionNormalTexture(new Vector3( 1000, 0,  1000), Vector3.Up, new Vector2(1, 1));
            OceanVD = new VertexDeclaration(Global.Graphics, VertexPositionNormalTexture.VertexElements);
        }

        public void Draw(GameTime gameTime, Camera cam, TextureCube skyTexture, Matrix proj)
        {
            // start the shader
            oceanEffect.Begin();
            oceanEffect.CurrentTechnique.Passes[0].Begin();

            // set the transforms
            oceanEffect.Parameters["World"].SetValue(Matrix.Identity);
            oceanEffect.Parameters["View"].SetValue(cam.GetViewMatrix());
            oceanEffect.Parameters["Projection"].SetValue(proj);
            oceanEffect.Parameters["EyePos"].SetValue(cam.Position);

            // choose and set the ocean textures
            int oceanTexIndex = ((int)(gameTime.TotalGameTime.TotalSeconds) % 4);
            oceanEffect.Parameters["normalTex"].SetValue(OceanNormalMaps[(oceanTexIndex + 1) % 4]);
            oceanEffect.Parameters["normalTex2"].SetValue(OceanNormalMaps[(oceanTexIndex) % 4]);
            oceanEffect.Parameters["textureLerp"].SetValue((((((float)gameTime.TotalGameTime.TotalSeconds) - (int)(gameTime.TotalGameTime.TotalSeconds)) * 2 - 1) * 0.5f) + 0.5f);

            // set the time used for moving waves
            oceanEffect.Parameters["time"].SetValue((float)gameTime.TotalGameTime.TotalSeconds * 0.02f);

            // set the sky texture
            oceanEffect.Parameters["cubeTex"].SetValue(skyTexture);

            oceanEffect.CommitChanges();

            // draw our geometry
            Global.Graphics.VertexDeclaration = OceanVD;
            Global.Graphics.DrawUserPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList, OceanVerts, 0, 2);

            // and we're done!
            oceanEffect.CurrentTechnique.Passes[0].End();
            oceanEffect.End();
        }
    }
}
